import React, { createContext, ReactNode, useReducer } from 'react';

// Interface definitions
interface IWarnings extends Document {
  text: string;
  date: Date;
  given_by: {
    _id: string;
    name: string;
  };
}

enum Role {
  Admin,
  Staff,
  User,
}

export interface IProfile {
  external_id: string;
  studentCardNumber?: string;
  roomNumber?: string;
  picture?: string;
  role: Role.Admin | Role.Staff | Role.User;
  email: string;
  name: string;
  warnings: [IWarnings] | [];
}

type Props = {
  children: ReactNode;
};

interface IContextProps {
  state: IProfile;
  dispatch: ({ type }: { type: string }) => void;
}

// Context
const initialState = {
  external_id: 'abcd',
  studentCardNumber: '1234',
  roomNumber: 104,
  picture: 'alma.jpg',
  role: Role.User,
  email: 'alma@gmail.com',
  name: 'Nagy Gizike',
  warning: [],
};
export const userContext = createContext({} as IContextProps);
const { Provider } = userContext;

export const UserStateProvider: React.FunctionComponent<Props> = (props: Props) => {
  const { children } = props;
  const [state, dispatch] = useReducer((prevState: any, action: any) => {
    switch (action.type) {
      case 'update':
        return action.payload;
      default:
        return prevState; // do nothing
    }
  }, initialState);

  return <Provider value={{ state, dispatch }}>{children}</Provider>;
};

export default { userContext, UserStateProvider };
